/**
 * Copyright 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 */
package com.sunlabs.archsim.base.hierarchy;

import com.sunlabs.archsim.engine.Director;

/**
 *
 * Base class for all entities modeled.
 * Covers basic roperies such as name, ID.
 * The class is not public as it is not intended to be used directly: 
 * Instead, derived classes are exposed as public.
 * 
 * @author ib27688
 */
public abstract class Entity {
	/** entity name */
	protected String name;

	/**
	 *  An entity may be known under an alias.
	 * TODO consider a set of aliases.
	 */
	protected String alias;
	

	/** the name used in the journal */
	protected String journalName;
	
	
	/** entity id, unique */
	protected long id;
	
	protected CompositeEntity parent;
	
	/** director */
	protected static Director director= Director.getInstance();
	
	/** collection of entities */
	
	/** make up the name from the id */
	protected Entity() {
		id= IDGenerator.newID();
		name= NameGenerator.newName(id);
		alias= name;
		journalName= alias;
		Entities.add(this);
		parent= null;
	} // Entity constructor
	
	protected Entity(String n) {
		name= n;
		alias= name;
		journalName= alias;
		id= IDGenerator.newID();
		Entities.add(this);
		parent= null;
	} // Entity constructor
	
	
	protected Entity(String name, CompositeEntity g) {
		this(name);
		parent= g;
	} // Entity constructor
	
	/** accessor for the name */
	public String getBaseName() {
		return name;
	} // getBaseName
	
	/** accessor for the alias */
	public String getBaseAlias() {
		return alias;
	} // getBaseAlias
	
	/** set the name */
	void setName(String n) {
		name= n;
	} // setName
	
	public void setAlias(String n) {
		// only change journal name if it was the same as the alias
		if (journalName.equals(alias)) journalName= n;
		alias= n;
	} // set alias
	
	public String getName() {
		if (parent == null) {
			return getBaseName();
		}
		else {
			return parent.getName() + "." + getBaseName();
		}
	} // getFullName
	
	public String getAlias() {
		if (parent == null) {
			return getBaseAlias();
		}
		else {
			return parent.getAlias() + "." + getBaseAlias();
		}		
	} // getAlias
	
	public void setJournalName(String n) {
		journalName= n;
	} // setJournalName
	
	public String getJournalName() {
		return journalName;
	} // getJournalName
	
	public void setParent(CompositeEntity p) {
		parent= p;
		// add yourself to the set of members if not already there
		if (!p.isMember(this)) {
			p.addMember(this);
		}
	} // setParent
	
	/** accessor for the id */
	public long getID() {
		return id;
	} //getID
	
	public String toString() {
		return name + "_" + id;
	}
	
	public abstract boolean selfCheck();
	
	public abstract void print();
	
	/** initalize the entity */
	public abstract void init();
	
}
